# frozen_string_literal: true

require 'json'
require 'net/http'

module Trefoil
  # Client used for making requests to the API. Respects API rules for rate limiting
  # and thread watching.
  class Client
    BASE_URL = 'a.4cdn.org'

    # Initialize with a last_request time 1 second prior to present
    # so that we can make requests right away
    def initialize
      @last_request = Time.now - 1
      @mutex = Mutex.new
      @board_cache = {}
    end

    # Get a JSON parsed response from a given endpoint. Limited to 1 request
    # per second in compliance with API rules.
    # @param endpoint [String] The endpoint to request data from.
    # @return [Hash<Symbol => String, Fixnum>] The returned data structure
    def get(endpoint)
      @mutex.synchronize do
        sleep time_until_available if must_wait?
        resp = Net::HTTP.get_response(BASE_URL, format_endpoint(endpoint))
        @last_request = Time.now
        case resp.code.to_i
        when 200
          JSON.parse(resp.body, symbolize_names: true)
        when 400..500
          raise 'Error making a request' # TODO: Better error handling
        else
          p endpoint
          raise 'Unhandled status code' # TODO: better code handling
        end
      end
    end

    # Get a board object based on its name
    # @param board_name [String]
    def board(board_name)
      Board.new(self, board_name)
    end

    private

    # Adds leading '/' to requests
    # @param endpoint [String] The endpoint being formatted
    # @return [String] `endpoint` with leading `/` added if needed
    def format_endpoint(endpoint)
      return endpoint if endpoint[0] == '/'

      '/' + endpoint
    end

    # Amount of time before the mutex is freed for the next request
    # @return [Fixnum, Float] Time remaining until the next request can be made
    def time_until_available
      (@last_request + 1) - Time.now
    end

    # Whether we must wait to make a request
    # @return [true, false] Whether we need to wait or not
    def must_wait?
      time_until_available.positive?
    end

    # Cache boards from `/boards.json`. Called when accessing `Board#[]` for
    # the first time. Effects last request time.
    def cache_boards
      boards = get('boards.json')
      boards[:boards].each do |board_obj|
        @board_cache[board_obj[:board]] = board_obj
      end

      @board_cache[:troll_flags] = boards[:troll_flags]
    end
  end
end
