# frozen_string_literal: true

require 'trefoil/board'

module Trefoil
  # A class representing an Image using data from a post object.
  class Image
    # @return [Integer] Renamed filename (UNIX timestamp + miliseconds)
    attr_reader :tim

    # @return [String] Original filename of this image
    attr_reader :filename

    # @return [String] File extension of the fullsize image.
    attr_reader :ext

    # @return [Integer] Filesize of the fullsize image.
    attr_reader :fsize

    # @return [String] MD5 hash of fullsize image.
    attr_reader :md5

    # @return [Integer] Width of fullsize image.
    attr_reader :w

    # @return [Integer] Height of fullsize image.
    attr_reader :h

    # @return [Integer] Width of thumbnail.
    attr_reader :tn_w

    # @return [Integer] Height of thumbnail.
    attr_reader :tn_h

    # @return [0, 1, nil] Whether this image is deleted. Use `#image?` for a boolean representation.
    attr_reader :filedeleted

    # @return [0, 1, nil] Whether this image is a spoiler. Use `#spoiler?` for a boolean representation.
    attr_reader :spoiler

    # @return [String] The name of the board this image is associated with.
    attr_reader :board_name

    # Create a new image object. Intended for internal use.
    # @param data [Hash<Symbol => Integer, String] Data from a post object to extrapolate info from.
    # @param board [Board, String] The board, or board name, associated with the post this image is from.
    def initialize(data, board)
      expand_data(data)
      @board_name = board.is_a?(Board) ? board[:name] : board
    end

    # The url to the full size image.
    # @return [String] url to the fullsize version of this image.
    def url
      "http://i.4cdn.org/#{board_name}/#{tim}#{ext}"
    end

    # The url to the thumbnail of this image.
    # @return [String] url to the thumbnail of this image.
    def thumbnail_url
      "http://i.4cdn.org/#{board_name}/#{tim}s.jpg"
    end

    # Convenience abstraction for determining if this file was deleted.
    # @return [true, false] Whether this file was deleted.
    def file_deleted?
      filedeleted == 1
    end

    # Convenience abstraction for determining if this file is a spoiler.
    # @return [true, false] Whether this file is a spoiler.
    def spoiler?
      spoiler == 1
    end

    private

    # Move desired values from post data into instance variables.
    # @param data [Hash<Symbol => Integer, String] Data from a Post object.
    def expand_data(data)
      %i[tim filename ext fsize md5 w h tn_w tn_h filedeleted spoiler].each do |sym|
        instance_variable_set("@#{sym}", data[sym])
      end
    end
  end
end
