# frozen_string_literal: true

module Trefoil
  # Represents a `post` object.
  # Post object data varies greatly based on board, content, and whether it is an OP.
  # Possible fields are available for viewing in the (API documentation)[https://github.com/4chan/4chan-API#posts-object]
  class Post
    # Intended for internal use
    def initialize(client, board, thread, data)
      @client = client
      @board = board
      @thread = thread
      @data = data
    end

    # Access underlying post data with symbolized names
    # Possible fields are available for viewing in the (API documentation)[https://github.com/4chan/4chan-API#posts-object]
    # @param key [Symbol] The API field
    # @return [Hash, Array, String, Fixnum]
    def [](key)
      @data[key]
    end

    # Whether this is the first post in a thread. All op's has a resto of 0
    # @return [true, false]
    def op?
      @data[:resto].zero?
    end

    # Whether this post has an image associated with it
    # @return [true, false]
    def image?
      !@data[:filename].nil?
    end

    # Image object of this post, if any.
    # @return [Image, nil] An Image or nil if no image is present.
    def image
      return nil unless image?

      Image.new(@data, @board)
    end

    # Url to this post
    # @return [String] The url to this post.
    def url
      direct_link = if @data[:resto].zero?
                      @data[:no]
                    else
                      "#{@data[:resto]}p##{@data[:no]}"
                    end

      "http://boards.4chan.org/#{@board}/thread/#{direct_link}"
    end

    # The OP to the thread this post is in.
    # @return [Post] A post object for the OP, the instance it was called on if it is the OP.
    def op
      if @data[:resto].zero
        self
      else
        thread.posts[0]
      end
    end
  end
end
